// Copyright 2003 Data Helper, Inc.     
// This program is realeased under the 
// GNU GENERAL PUBLIC LICENSE Version 2.
// See COPYING.GPL for information.    
// Mark Willcox <pops@datahelper.com> 

// Run this before qmail-smtpd. If the IP address has been 
// vetted by postpop, we'll set RELAYCLIENT.
// If anything is wrong, just go on without setting RELAYCLIENT.

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

// The following are defined at compile time:
//   IP_DIR     The directory were the IP address files go.
//   AUTH_LIFE  Number of seconds that the door remains open.
//   DIR_DEPTH  Number of nodes to use as subdirectores.

int main(int argc, char **argv)
{
    if (argc < 2) 
        _exit( 2 );

    checkAuth();	// Sets env var RELAYCLIENT if OK.`

    execvp(argv[1],argv + 1);
    _exit(111);
   
    return 0;
}

int checkAuth()
{
    int     dot;
    char    ip[16];
    char    *lastDot;
    char    fn[PATH_MAX];
    struct stat finfo;

   
    if (! getenv( "TCPREMOTEIP"))
        return;

    // Make sure IP is a reasonable size.  Save it.
    if (strlen( getenv( "TCPREMOTEIP")) > 15)
        return;
    strncpy( ip, getenv( "TCPREMOTEIP" ), 16 );
    if (strlen( ip ) < 7)
        return;

    strcpy( fn, IP_DIR );

    for (dot = 0; dot < DIR_DEPTH; ++dot)
    {
        lastDot = strrchr( ip, '.' );
        if (! lastDot)
            return;

        *lastDot++ = '\0';
        strcat( fn, lastDot );
        strcat( fn, "/" );
    }

    strcat( fn, ip );

    if (stat( fn, &finfo ) == -1 )
        return;

    if ((time( NULL ) - finfo.st_mtime) > AUTH_LIFE )
        return;

    putenv( "RELAYCLIENT=" );
}
